//Canvas Class

#include "CanvasClass.h"

//This is for standalone canvas with no LoadFromFile() option
CanvasClass::CanvasClass(HWND hwnd,int WIDTH,int HEIGHT){
    STX=0; STY=0;
    WID=WIDTH; HEI=HEIGHT;
    OwnerHwnd=hwnd;
    
    HDC hdc;
    hdc=GetDC(OwnerHwnd);
    if(!hdc)MessageBox(OwnerHwnd, TEXT("ERROR: GetDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
    Mhdc =  CreateCompatibleDC ( hdc );
    if(!Mhdc)MessageBox(OwnerHwnd, TEXT("ERROR: CreateCompatibleDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	Mhbmp = CreateCompatibleBitmap ( hdc,WID,HEI );
    if(!Mhbmp)MessageBox(OwnerHwnd, TEXT("ERROR: CreateCompatibleBitmap() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	SelectObject ( Mhdc, Mhbmp );
    ReleaseDC(OwnerHwnd,hdc);
    
    Init();
    EnableRefresh=true;
}

//This is for standalone canvas with no LoadFromFile() option; special cases X,Y,WID,HEI
CanvasClass::CanvasClass(HWND hwnd,int STARTX,int STARTY,int WIDTH,int HEIGHT,char mode){
    STX=STARTX; STY=STARTY;
    WID=WIDTH; HEI=HEIGHT;
    OwnerHwnd=hwnd;
    
    HDC hdc;
    hdc=GetDC(OwnerHwnd);
    if(!hdc)MessageBox(OwnerHwnd, TEXT("ERROR: GetDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
    Mhdc =  CreateCompatibleDC ( hdc );
    if(!Mhdc)MessageBox(OwnerHwnd, TEXT("ERROR: CreateCompatibleDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	Mhbmp = CreateCompatibleBitmap ( hdc,WID,HEI );
    if(!Mhbmp)MessageBox(OwnerHwnd, TEXT("ERROR: CreateCompatibleBitmap() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	SelectObject ( Mhdc, Mhbmp );
    ReleaseDC(OwnerHwnd,hdc);
    
    Init();
    EnableRefresh=true;
}

//This is for Bitmap image that can be loaded from file: LoadFromFile()
CanvasClass::CanvasClass(HWND hwnd,int STARTX,int STARTY,int WIDTH,int HEIGHT){
    STX=STARTX; STY=STARTY;
    WID=WIDTH; HEI=HEIGHT;
    OwnerHwnd=hwnd;
    
    Init();
    EnableRefresh=false;
}

CanvasClass::~CanvasClass(){
    DeleteDC(Mhdc);
    DeleteObject(Mhbmp);
    delete Brush;
    delete Pen;
}

void CanvasClass::Init(){
    Pixels.PixRow.OwnerHDC=&Mhdc;
    hBrush=NULL;
    hPen=NULL;
    Brush=new BrushClass();
    Pen=new PenClass();
    CurX=0;CurY=0;
}

void CanvasClass::Repaint(){
  	HDC         hdc;
    PAINTSTRUCT ps;
    RECT        rect;
    hdc = BeginPaint (OwnerHwnd, &ps);  // Get DC handle
    if(!hdc)MessageBox(OwnerHwnd, TEXT("ERROR: BeginPaint() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	//GetClientRect (OwnerHwnd, &rect);   // Get client rectangle
	if(EnableRefresh){
	    if(!(BitBlt(hdc,STX,STY,WID,HEI,Mhdc,0, 0, SRCCOPY)))
    	    MessageBox(OwnerHwnd, TEXT("ERROR: BitBlt() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);
	}
    EndPaint (OwnerHwnd, &ps);          // Release DC
}

void CanvasClass::LoadFromFile(AnsiString filename){
     LoadFromFile(filename.c_str());
}

void CanvasClass::LoadFromFile(char* filename){
    HDC hdc;
    hdc=GetDC(OwnerHwnd);
    if(!hdc){MessageBox(OwnerHwnd, TEXT("ERROR: GetDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);return;}
    
    Mhbmp=(HBITMAP)LoadImage(NULL,filename,IMAGE_BITMAP,WID,HEI,LR_LOADFROMFILE);
    //Mhbmp=(HBITMAP)LoadImage(NULL,"Push-pull3.bmp",IMAGE_BITMAP,WID,HEI,LR_LOADFROMFILE);
    
    if(!Mhbmp){MessageBox(OwnerHwnd, TEXT("ERROR: LoadImage() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);return;}
	Mhdc=CreateCompatibleDC(hdc);
	if(!Mhdc){MessageBox(OwnerHwnd, TEXT("ERROR: CreateCompatibleDC() failed !"), TEXT("Bitmap error"), MB_APPLMODAL | MB_ICONSTOP);return;}
	SelectObject ( Mhdc, Mhbmp );
    ReleaseDC(OwnerHwnd,hdc);
    EnableRefresh=true;
    Repaint();
	InvalidateRect(OwnerHwnd,NULL,TRUE);
}

void CanvasClass::SetPix(int pix,int xIndex,int yIndex){
    SetPixel(Mhdc,xIndex,yIndex,RGB( (pix&0xFF),(pix&0xFF00)>>8,(pix&0xFF0000)>>16 ));
}

int CanvasClass::GetPix(int xIndex,int yIndex){
    int pix;
    pix=GetPixel(Mhdc,xIndex,yIndex);
    return pix;
}

void CanvasClass::FillRect(RECT r){
    int C;
    C=Brush->Color;
    hBrush = CreateSolidBrush(RGB( (C&0xFF),(C&0xFF00)>>8,(C&0xFF0000)>>16 ) );
    //SelectObject(Mhdc, hPen);
    //SetBkColor(hdc, RGB(0,0,0));
    //Rectangle(Mhdc, r.left,r.top,r.right,r.bottom);
    Brush->DrawFillRect(Mhdc,&r,hBrush);
    DeleteObject(hBrush);
}

void CanvasClass::MoveTo(int x,int y){
    CurX=x;CurY=y;
}

void CanvasClass::LineTo(int x,int y){
    int C=Pen->Color;
    hPen = CreatePen(PS_SOLID,1,RGB( (C&0xFF),(C&0xFF00)>>8,(C&0xFF0000)>>16 ) );
    SelectObject(Mhdc, hPen);
     
    MoveToEx(Mhdc, CurX, CurY, (LPPOINT) NULL);
    Pen->LineToDraw(Mhdc,x,y);
     
    DeleteObject(hPen);
	CurX=x;CurY=y;
}

void CanvasClass::Update(){
	RECT r;
	r.left=STX;
	r.top=STY;
	r.right=STX+WID;
	r.bottom=STY+HEI;
	InvalidateRect(OwnerHwnd,&r,FALSE);
	Repaint();
}
