//NetworkIterface

#include "NetworkIterface.h"

#define SERVERMODE true
#define CLIENTMODE false

__fastcall TNetworkIface::TNetworkIface(bool IsServerMode, AnsiString AIP, AnsiString APort ) : Thread(true){
           
//   AIP: hostname , IP address or hostname as "lxlogin.redback.com"
//   For server AIP ios don't care, for client it is hostname

    ServerMode = IsServerMode;
    ListenSocket = INVALID_SOCKET;
    ClientSocket = INVALID_SOCKET;
    result = NULL;
    ptr = NULL;
    recvbuflen = DEFAULT_BUFLEN;
    recvbuf=new char[DEFAULT_BUFLEN];
    
    char emsg[150];
    connected=false;
    
    // Initialize Winsock
    iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
    if (iResult != 0) {
        sprintf(emsg,"WSAStartup failed with error: %d", iResult);
        SocketErrorHandler(1,emsg);
        return;
    }
    ZeroMemory(&hints, sizeof(hints));
    
    if(ServerMode){
    //SERVER
        hints.ai_family = AF_INET;
        hints.ai_socktype = SOCK_STREAM;
        hints.ai_protocol = IPPROTO_TCP;
        hints.ai_flags = AI_PASSIVE;
        // Resolve the server address and port
        iResult = getaddrinfo(NULL, APort.c_str(), &hints, &result);
        if ( iResult != 0 ) {
            sprintf(emsg,"getaddrinfo failed with error: %d", iResult);
            SocketErrorHandler(2,emsg);
            return;
        }
        // Create a SOCKET for connecting to server
        ListenSocket = socket(result->ai_family, result->ai_socktype, result->ai_protocol);
        if (ListenSocket == INVALID_SOCKET) {
            sprintf(emsg,"socket failed with error: %ld", WSAGetLastError());
            freeaddrinfo(result);
            SocketErrorHandler(3,emsg);
            return;
        }
        // Setup the TCP listening socket
        iResult = bind( ListenSocket, result->ai_addr, (int)result->ai_addrlen);
        if (iResult == SOCKET_ERROR) {
            sprintf(emsg,"bind failed with error: %d", WSAGetLastError());
            freeaddrinfo(result);
            closesocket(ListenSocket);
            SocketErrorHandler(4,emsg);
            return;
        }
        freeaddrinfo(result);
        
        Resume();
    
    }else{
    //CLIENT
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;
        hints.ai_protocol = IPPROTO_TCP;
        // Resolve the server address and port
        iResult = getaddrinfo(AIP.c_str(), APort.c_str(), &hints, &result);
        if ( iResult != 0 ) {
            sprintf(emsg,"getaddrinfo failed with error: %d", iResult);
            WSACleanup();
            SocketErrorHandler(20,emsg);
            return;
        }
        
        // Attempt to connect to an address until one succeeds
        for(ptr=result; ptr != NULL ;ptr=ptr->ai_next) {
            // Create a SOCKET for connecting to server
            ClientSocket = socket(ptr->ai_family, ptr->ai_socktype, 
                ptr->ai_protocol);
            if (ClientSocket == INVALID_SOCKET) {
                sprintf(emsg,"socket failed with error: %ld", WSAGetLastError());
                WSACleanup();
                SocketErrorHandler(21,emsg);
                return;
            }
            // Connect to server.
            iResult = connect( ClientSocket, ptr->ai_addr, (int)ptr->ai_addrlen);
            if (iResult == SOCKET_ERROR) {
                closesocket(ClientSocket);
                ClientSocket = INVALID_SOCKET;
                continue;
            }
            break;
        }
        
        freeaddrinfo(result);
        
        if (ClientSocket == INVALID_SOCKET) {
            sprintf(emsg,"Unable to connect to server!");
            WSACleanup();
            SocketErrorHandler(22,emsg);
            return;
        }
                
        Resume();

    }
    
}

__fastcall TNetworkIface::~TNetworkIface(){
    delete[] recvbuf;
    WSACleanup();
    Terminate();
    //WaitForMultipleObjects(1, &HThread, TRUE, INFINITE);
    WaitForMultipleObjects(1, &HThread, TRUE, 2000);
}

void __fastcall TNetworkIface::SocketErrorHandler( int ErrCode, char* errMSG ){
    if(ErrCode!=1){
        Terminate();
        WSACleanup();
    }
    connected=false;
    AnsiString str(errMSG);
    OnNetworkErrorEvent(ErrCode,str);
}

void __fastcall TNetworkIface::Execute(){
    char emsg[150];
    while(!Terminated){
        MsgWaitForMultipleObjectsEx(0,NULL,10,0,0);
        if(ServerMode){
        //SERVER
            if(!connected){
                //Listening...
                iResult = listen(ListenSocket, SOMAXCONN);
                if (iResult == SOCKET_ERROR) {
                    sprintf(emsg,"listen failed with error: %d", WSAGetLastError());
                    closesocket(ListenSocket);
                    Terminate();
                    WSACleanup();
                    SocketErrorHandler(5,emsg);
                    return;
                }
                // Accept a client socket
                ClientSocket = accept(ListenSocket, NULL, NULL);
                if (ClientSocket == INVALID_SOCKET) {
                    sprintf(emsg,"accept failed with error: %d", WSAGetLastError());
                    closesocket(ListenSocket);
                    Terminate();
                    WSACleanup();
                    SocketErrorHandler(6,emsg);
                    return;
                }
                // No longer need server socket
                closesocket(ListenSocket);
                connected=true;            
                PostMessage(OwnerHwnd,WM_COMMAND,MSG_NETWORKCONNECTED,0);
                
            }else{
                  
                //Receive data from client
                iResult = recv(ClientSocket, recvbuf, recvbuflen, 0);
                if (iResult > 0) {
                    OnReceiveDataEvent(recvbuf, iResult);
                    recvbuf[AnsiStringMAXLEN-1]=0;
                    recvbuf[iResult]=0;
                    AnsiString str(recvbuf);
                    OnReceiveStringEvent(str);
                } else if (iResult == 0) {                       
                    connected=false;
                    // shutdown the connection since we're done
                    iResult = shutdown(ClientSocket, SD_SEND);
                    if (iResult == SOCKET_ERROR) {
                        sprintf(emsg,"shutdown failed with error: %d", WSAGetLastError());
                        closesocket(ClientSocket);
                        SocketErrorHandler(11,emsg);
                    }                    
                    // cleanup
                    closesocket(ClientSocket);
                    Terminate();                    
                    WSACleanup();
                    
                    PostMessage(OwnerHwnd,WM_COMMAND,MSG_NETWORKCLOSED,0);
                                        
                    return;
                } else  {
                    sprintf(emsg,"recv failed with error: %d", WSAGetLastError());
                    closesocket(ClientSocket);
                    Terminate();
                    WSACleanup();
                    SocketErrorHandler(10,emsg);
                    return;
                }

            }
        }else{
        //CLIENT
        
            iResult = recv(ClientSocket, recvbuf, recvbuflen, 0);
            if ( iResult > 0 ){
                OnReceiveDataEvent(recvbuf, iResult);
                recvbuf[AnsiStringMAXLEN-1]=0;
                recvbuf[iResult]=0;
                AnsiString str(recvbuf);
                OnReceiveStringEvent(str);                 
            }else{ 
                int TMPiResult=iResult;
                // shutdown the connection since no more data will be sent
                iResult = shutdown(ClientSocket, SD_SEND);
                if (iResult == SOCKET_ERROR) {
                    sprintf(emsg,"shutdown failed with error: %d\n", WSAGetLastError());
                    closesocket(ClientSocket);
                    WSACleanup();
                    Terminate();
                    SocketErrorHandler(24,emsg);
                    return;
                }
                // cleanup
                closesocket(ClientSocket);
                Terminate();
                WSACleanup();
                if ( TMPiResult == 0 ){
                    PostMessage(OwnerHwnd,WM_COMMAND,MSG_NETWORKCLOSED,0);
                }else{
                    sprintf(emsg,"recv failed with error: %d\n", WSAGetLastError());
                    SocketErrorHandler(29,emsg);
                }
            }
            

        }
    
    }
    
    connected=false;
}

bool __fastcall TNetworkIface::SendData( int ASize, char* AData){
    char emsg[150];
    //Send Data
    if(ServerMode&&(!connected)){
        SocketErrorHandler(8,"No client connected");
        return false;
    }
    if(ClientSocket==INVALID_SOCKET){
        SocketErrorHandler(18,"ClientSocket is invalid");
        return false;
    }
    iSendResult = send( ClientSocket, AData, ASize, 0 );
    if (iSendResult == SOCKET_ERROR) {
        sprintf(emsg,"send failed with error: %d", WSAGetLastError());
        closesocket(ClientSocket);
        WSACleanup();        
        Terminate();
        SocketErrorHandler(10,emsg);
        return true;
    }      
}
